/******************************************************************************

                               Copyright (c) 2007
                            Infineon Technologies AG
                     Am Campeon 1-12; 81726 Munich, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

******************************************************************************/

#define DSL_INTERN

#include "drv_dsl_cpe_api.h"

#if defined(INCLUDE_DSL_CPE_API_VINAX) && defined(INCLUDE_DSL_PM)

#include "drv_dsl_cpe_pm_core.h"

#undef DSL_DBG_BLOCK
#define DSL_DBG_BLOCK DSL_DBG_PM

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_Start(DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   /* Reset bPmDataValid flag*/
   DSL_DRV_PM_CONTEXT(pContext)->bPmDataValid = DSL_FALSE;

   /* Reset FW mode information*/
   DSL_DRV_PM_CONTEXT(pContext)->nLastShowtime = DSL_PM_FWMODE_NA;
   DSL_DRV_PM_CONTEXT(pContext)->nCurrShowtime = DSL_PM_FWMODE_NA;

   /* not for ADSL-only mode */
   if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
   {
      nErrCode = DSL_DRV_VNX_SendMsgTr1Control(
         pContext, DSL_FALSE, DSL_FALSE, 0xFFFF, CMD_TR1ControlSet_NOT_USED, 0);
   }

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_Restart(DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   /* not for ADSL-only mode */
   if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
   {
      nErrCode = DSL_DRV_VNX_SendMsgTr1Control(
         pContext, DSL_FALSE, DSL_FALSE, 0xFFFF, CMD_TR1ControlSet_NOT_USED, 0);
   }

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_Suspend(DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   /* Reset bPmDataValid flag*/
   DSL_DRV_PM_CONTEXT(pContext)->bPmDataValid = DSL_FALSE;

   /* Update nLastShowtime value*/
   DSL_DRV_PM_CONTEXT(pContext)->nLastShowtime = DSL_DRV_PM_CONTEXT(pContext)->nCurrShowtime;

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_OnEapsTimeout(DSL_Context_t *pContext)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_PM_Reset_t nPmResetType;

   if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) ||
       DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1))
   {
      DSL_DRV_PM_CONTEXT(pContext)->nCurrShowtime = DSL_PM_FWMODE_VDSL;
   }
   else if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) ||
            DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB))
   {
      DSL_DRV_PM_CONTEXT(pContext)->nCurrShowtime = DSL_PM_FWMODE_ADSL;
   }
   else
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Unknown FW mode!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
      DSL_DRV_PM_CONTEXT(pContext)->nCurrShowtime = DSL_PM_FWMODE_NA;
   }

   if (DSL_DRV_PM_CONTEXT(pContext)->nCurrShowtime != DSL_DRV_PM_CONTEXT(pContext)->nLastShowtime)
   {
      /* Reset all PM module counters*/
      nPmResetType.data.nResetType = DSL_PM_RESET_ALL;
      nErrCode = DSL_DRV_PM_Reset(pContext, &nPmResetType);
      
      if (nErrCode != DSL_SUCCESS)
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - PM counters reset failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
         return nErrCode;
      }

      /* Reset Internal PM counters state*/
      DSL_DRV_PM_InternalStateReset(pContext);

      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: Permanent xDSL FW mode switch detected, all PM counters are reset..."
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
   }

   /* Set bPmDataValid flag*/
   DSL_DRV_PM_CONTEXT(pContext)->bPmDataValid = DSL_TRUE;

   return nErrCode;
}

#ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS
/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_ChannelCountersGet(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   DSL_XTUDir_t nDirection,
   DSL_PM_ChannelData_t *pCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;

   union
   {
      CMD_CRC_StatsNE_NoTR1Get_t ADSL_CrcStatsNE;
      CMD_FEC_StatsNE_NoTR1Get_t ADSL_FecStatsNE;
      /* CMDs for the NE and FE have the same structure*/
      CMD_CRC_StatsNE_Get_t VDSL_CrcStatsNE;
      CMD_CRC_StatsFE_Get_t VDSL_CrcStatsFE;
      /* CMDs for the NE and FE have the same structure*/
      CMD_FEC_StatsNE_Get_t VDSL_FecStatsNE;
      CMD_FEC_StatsFE_Get_t VDSL_FecStatsFE;
   } sCmd;

   union
   {
      ACK_CRC_StatsNE_NoTR1Get_t ADSL_CrcStatsNE;
      ACK_FEC_StatsNE_NoTR1Get_t ADSL_FecStatsNE;
      /* ACKs for the NE and FE have the same structure*/
      ACK_CRC_StatsNE_Get_t VDSL_CrcStatsNE;
      ACK_CRC_StatsFE_Get_t VDSL_CrcStatsFE;
      /* ACKs for the NE and FE have the same structure*/
      ACK_FEC_StatsNE_Get_t VDSL_FecStatsNE;
      ACK_FEC_StatsFE_Get_t VDSL_FecStatsFE;
   } sAck;

   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_ATU_DIRECTION(nDirection);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DEV_ChannelCountersCurrentGet, (nDirection=%s, nChannel=%d)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nDirection==DSL_NEAR_END ? "NE":"FE", nChannel));

   /* Reset the output structure*/
   memset(pCounters, 0x0, sizeof(DSL_PM_ChannelData_t));

   /* not for ADSL, Far-End mode */
   if( !DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) && nDirection == DSL_FAR_END )
   {
      DSL_DEBUG( DSL_DBG_WRN,
         (pContext, "DSL[%02d]: WARNING - Channel counters are not supported for the ADSL FE!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_SUCCESS;
   }

   if( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
   {
   /*... VDSL only mode*/
      memset(&(sAck.VDSL_CrcStatsNE), 0x0, sizeof(sAck.VDSL_CrcStatsNE));

      sCmd.VDSL_CrcStatsFE.Index  = sCmd.VDSL_CrcStatsNE.Index  = 0x0;
      sCmd.VDSL_CrcStatsFE.Length = sCmd.VDSL_CrcStatsNE.Length = 8;

      /* Get Code Violations*/
      nErrCode = DSL_DRV_VNX_SendMessage(
         pContext,
         nDirection == DSL_NEAR_END ? CMD_CRC_STATSNE_GET: CMD_CRC_STATSFE_GET,
         sizeof(sCmd.VDSL_CrcStatsNE), (DSL_uint8_t*)&(sCmd.VDSL_CrcStatsNE),
         sizeof(sAck.VDSL_CrcStatsNE), (DSL_uint8_t*)&(sAck.VDSL_CrcStatsNE));

      /* Check nErrCode and return on error*/
      if( nErrCode < 0 )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Code Violations statistics Get failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

      /* ACKs for the CMD_CRC_STATSNE_GET and CMD_CRC_STATSFE_GET have the
          same strucrure, so we can use any*/
      pCounters->nCodeViolations = (DSL_uint32_t)sAck.VDSL_CrcStatsNE.cntCVI_LSW |
                                 (((DSL_uint32_t)sAck.VDSL_CrcStatsNE.cntCVI_MSW) << 16);

      memset(&(sAck.VDSL_FecStatsNE), 0x0, sizeof(sAck.VDSL_FecStatsNE));

      sCmd.VDSL_FecStatsFE.Index  = sCmd.VDSL_FecStatsNE.Index  = 0x0;
      sCmd.VDSL_FecStatsFE.Length = sCmd.VDSL_FecStatsNE.Length = 24;

      /* Get Forward Error Corrections*/
      nErrCode = DSL_DRV_VNX_SendMessage(
         pContext,
         nDirection == DSL_NEAR_END ? CMD_FEC_STATSNE_GET: CMD_FEC_STATSFE_GET,
         sizeof(sCmd.VDSL_FecStatsNE), (DSL_uint8_t*)&(sCmd.VDSL_FecStatsNE),
         sizeof(sAck.VDSL_FecStatsNE), (DSL_uint8_t*)&(sAck.VDSL_FecStatsNE));

      /* Check nErrCode and return on error*/
      if( nErrCode < 0 )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Forward Error Corrections statistics Get failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

      /* ACKs for the CMD_FEC_STATSNE_GET and CMD_FEC_STATSFE_GET have the
          same strucrure, so we can use any*/
      pCounters->nFEC = (DSL_uint32_t)sAck.VDSL_FecStatsNE.cntECI_LSW |
                      (((DSL_uint32_t)sAck.VDSL_FecStatsNE.cntECI_MSW) << 16);
   }
   else if(DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) ||
           DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB))
   {
   /*... ADSL only mode*/
      memset(&(sAck.ADSL_CrcStatsNE), 0x0, sizeof(sAck.ADSL_CrcStatsNE));

      sCmd.ADSL_CrcStatsNE.Index  = 0x0;
      sCmd.ADSL_CrcStatsNE.Length = 4;

      /* Get Code Violations*/
      nErrCode = DSL_DRV_VNX_SendMessage(
         pContext, CMD_CRC_STATSNE_NOTR1GET,
         sizeof(sCmd.ADSL_CrcStatsNE), (DSL_uint8_t*)&(sCmd.ADSL_CrcStatsNE),
         sizeof(sAck.ADSL_CrcStatsNE), (DSL_uint8_t*)&(sAck.ADSL_CrcStatsNE));

      /* Check nErrCode and return on error*/
      if( nErrCode < 0 )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Code Violations statistics get failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

      /* ACKs for the CMD_CRC_STATSNE_GET and CMD_CRC_STATSFE_GET have the
          same strucrure, so we can use any*/
      pCounters->nCodeViolations = (DSL_uint32_t)sAck.ADSL_CrcStatsNE.cntCVI_LSW |
                                 (((DSL_uint32_t)sAck.ADSL_CrcStatsNE.cntCVI_MSW) << 16);

      memset(&(sAck.ADSL_FecStatsNE), 0x0, sizeof(sAck.ADSL_FecStatsNE));

      sCmd.ADSL_FecStatsNE.Index  = 0x0;
      sCmd.ADSL_FecStatsNE.Length = 12;

      /* Get Forward Error Corrections*/
      nErrCode = DSL_DRV_VNX_SendMessage(
         pContext, CMD_FEC_STATSNE_NOTR1GET,
         sizeof(sCmd.ADSL_FecStatsNE), (DSL_uint8_t*)&(sCmd.ADSL_FecStatsNE),
         sizeof(sAck.ADSL_FecStatsNE), (DSL_uint8_t*)&(sAck.ADSL_FecStatsNE));

      /* Check nErrCode and return on error*/
      if( nErrCode < 0 )
      {
         DSL_DEBUG( DSL_DBG_ERR,
            (pContext, "DSL[%02d]: ERROR - Forward Error Corrections statistics get failed!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
            
         return nErrCode;
      }

      /* ACKs for the CMD_FEC_STATSNE_GET and CMD_FEC_STATSFE_GET have the
          same strucrure, so we can use any*/
      pCounters->nFEC = (DSL_uint32_t)sAck.ADSL_FecStatsNE.cntECI_LSW |
                      (((DSL_uint32_t)sAck.ADSL_FecStatsNE.cntECI_MSW) << 16);
   }
   else
   {
     DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Unknown operation mode!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      nErrCode = DSL_ERROR;
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_PM_DEV_ChannelCountersCurrentGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_ChannelCountersSet(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   DSL_XTUDir_t nDirection,
   DSL_PM_ChannelData_t *pCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_LineStateValue_t nLineState = DSL_LINESTATE_UNKNOWN;

   union
   {
      CMD_CRC_StatsNE_NoTR1Set_t CrcStatsNEadsl;
      CMD_FEC_StatsNE_NoTR1Set_t FecStatsNEadsl;
      /* Near-End and Far-End has the same structures*/
      CMD_CRC_StatsNE_Set_t      CrcStatsNEvdsl;
      CMD_CRC_StatsFE_Set_t      CrcStatsFEvdsl;
      /* Near-End and Far-End has the same structures*/
      CMD_FEC_StatsNE_Set_t      FecStatsNEvdsl;
      CMD_FEC_StatsFE_Set_t      FecStatsFEvdsl;
   } sCmd;

   union
   {
      ACK_CRC_StatsNE_NoTR1Set_t CrcStatsNEadsl;
      ACK_FEC_StatsNE_NoTR1Set_t FecStatsNEadsl;
      ACK_CRC_StatsNE_Set_t      CrcStatsNEvdsl;
      ACK_CRC_StatsFE_Set_t      CrcStatsFEvdsl;
      ACK_FEC_StatsNE_Set_t      FecStatsNEvdsl;
      ACK_FEC_StatsFE_Set_t      FecStatsFEvdsl;
   } sAck;

   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DEV_ChannelCountersSet, (nDirection=%s, nChannel=%d)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nDirection==DSL_NEAR_END ? "NE":"FE",nChannel));

   /* Restore counters only if the last showtime FW mode equals to the
      current loaded FW binary*/
   if( ((DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) ||
         DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1)) &&
        (DSL_DRV_PM_CONTEXT(pContext)->nLastShowtime == DSL_PM_FWMODE_VDSL)) ||
       ((DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) ||
         DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB)) &&
        (DSL_DRV_PM_CONTEXT(pContext)->nLastShowtime == DSL_PM_FWMODE_ADSL)) )
   {
      /* Check line state*/
      if( DSL_DRV_DEV_LineStateGet(pContext, &nLineState) != DSL_ERROR )
      {
         if( nLineState != DSL_LINESTATE_IDLE )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - PM channel counters restore only allowed "
               "in the IDLE state"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
            return DSL_ERROR;
         }
      }

      if( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
      {
         /* Near-End and Far-End has the same structures*/
         memset(&(sCmd.CrcStatsNEvdsl), 0x0, sizeof(sCmd.CrcStatsNEvdsl));

         /* Fill the message length parameter*/
         sCmd.CrcStatsNEvdsl.Length = 8;

         /* Fill the LSW of the nCodeViolations counter*/
         sCmd.CrcStatsNEvdsl.cntCVI_LSW = (DSL_uint16_t)(pCounters->nCodeViolations & 0xFFFF);
         /* Fill the MSW of the nCodeViolations counter*/
         sCmd.CrcStatsNEvdsl.cntCVI_MSW = (DSL_uint16_t)((pCounters->nCodeViolations >> 16) & 0xFFFF);

         /* Set Code Violations*/
         nErrCode = DSL_DRV_VNX_SendMessage(
            pContext, 
            nDirection == DSL_NEAR_END ? CMD_CRC_STATSNE_SET : CMD_CRC_STATSFE_SET,
            sizeof(sCmd.CrcStatsNEvdsl), (DSL_uint8_t*)&(sCmd.CrcStatsNEvdsl),
            sizeof(sAck.CrcStatsNEvdsl), (DSL_uint8_t*)&(sAck.CrcStatsNEvdsl));

         if( nErrCode < 0 )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - CRC statistics Set failed!" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
            
            return nErrCode;
         }

         memset(&(sCmd.FecStatsNEvdsl), 0x0, sizeof(sCmd.FecStatsNEvdsl));

         /* Fill the message length parameter*/
         sCmd.FecStatsNEvdsl.Length = 24;

         /* Fill the LSW of the nFEC counter*/
         sCmd.FecStatsNEvdsl.cntECI_LSW = (DSL_uint16_t)(pCounters->nFEC & 0xFFFF);
         /* Fill the MSW of the nFEC counter*/
         sCmd.FecStatsNEvdsl.cntECI_MSW = (DSL_uint16_t)((pCounters->nFEC >> 16) & 0xFFFF);

         /* Set FEC*/
         nErrCode = DSL_DRV_VNX_SendMessage(
            pContext,
            nDirection == DSL_NEAR_END ? CMD_FEC_STATSNE_SET : CMD_FEC_STATSFE_SET,
            sizeof(sCmd.FecStatsNEvdsl), (DSL_uint8_t*)&(sCmd.FecStatsNEvdsl),
            sizeof(sAck.FecStatsNEvdsl), (DSL_uint8_t*)&(sAck.FecStatsNEvdsl));

         if( nErrCode < 0 )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - Forward Error Corrections statistics Set failed!" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
            
            return nErrCode;
         }
      }
      else
      {
         if( nDirection == DSL_NEAR_END )
         {
            memset(&(sCmd.CrcStatsNEadsl), 0x0, sizeof(sCmd.CrcStatsNEadsl));

            /* Fill the message length parameter*/
            sCmd.CrcStatsNEadsl.Length = 4;

            /* Fill the LSW of the nCodeViolations counter*/
            sCmd.CrcStatsNEadsl.cntCVI_LSW = (DSL_uint16_t)(pCounters->nCodeViolations & 0xFFFF);
            /* Fill the MSW of the nCodeViolations counter*/
            sCmd.CrcStatsNEadsl.cntCVI_MSW = (DSL_uint16_t)((pCounters->nCodeViolations >> 16) & 0xFFFF);

            /* Set Code Violations*/
            nErrCode = DSL_DRV_VNX_SendMessage(
               pContext, CMD_CRC_STATSNE_NOTR1SET,
               sizeof(sCmd.CrcStatsNEadsl), (DSL_uint8_t*)&(sCmd.CrcStatsNEadsl),
               sizeof(sAck.CrcStatsNEadsl), (DSL_uint8_t*)&(sAck.CrcStatsNEadsl));

            if( nErrCode != DSL_SUCCESS )
            {
               DSL_DEBUG( DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: ERROR - Code Violations statistics Set failed!" DSL_DRV_CRLF,
                  DSL_DEV_NUM(pContext)));
               
               return nErrCode;
            }

            memset(&(sCmd.FecStatsNEadsl), 0x0, sizeof(sCmd.FecStatsNEadsl));

            /* Fill the message length parameter*/
            sCmd.FecStatsNEadsl.Length = 12;

            /* Fill the LSW of the nFEC counter*/
            sCmd.FecStatsNEadsl.cntECI_LSW = (DSL_uint16_t)(pCounters->nFEC & 0xFFFF);
            /* Fill the MSW of the nFEC counter*/
            sCmd.FecStatsNEadsl.cntECI_MSW = (DSL_uint16_t)((pCounters->nFEC >> 16) & 0xFFFF);

            /* Set FEC*/
            nErrCode = DSL_DRV_VNX_SendMessage(
               pContext, CMD_FEC_STATSNE_NOTR1SET,
               sizeof(sCmd.FecStatsNEadsl), (DSL_uint8_t*)&(sCmd.FecStatsNEadsl),
               sizeof(sAck.FecStatsNEadsl), (DSL_uint8_t*)&(sAck.FecStatsNEadsl));

            if( nErrCode < 0 )
            {
               DSL_DEBUG( DSL_DBG_ERR,
                  (pContext, "DSL[%02d]: ERROR - Forward Error Corrections statistics Set failed!"
                  DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
               
               return nErrCode;
            }
         }
      }
   }
   else
   {
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: FW mode switch detected, PM channel counters restore skipped..."
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
   }
   
   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_DEV_ChannelCountersSet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}
#endif /* #ifdef INCLUDE_DSL_CPE_PM_CHANNEL_COUNTERS*/

#ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS
/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_DataPathCountersGet(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   DSL_XTUDir_t nDirection,
   DSL_PM_DataPathData_t *pCounters)
{
   DSL_Error_t    nErrCode = DSL_SUCCESS;

   union
   {
      CMD_G997_PTM_StatsGet_t   cmdG997PtmStatsGet;
      CMD_G997_ATM_StatsGet_t   cmdG997AtmStatsGet;
   } sCmd;

   union
   {
      ACK_G997_PTM_StatsGet_t   ackG997PtmStatsGet;
      ACK_G997_ATM_StatsGet_t   ackG997AtmStatsGet;
   } sAck;


   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_ERR_CODE();

   if( nChannel > 1 )
      return DSL_SUCCESS;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DEV_DataPathCountersCurrentGet, (nDirection=%s, nChannel=%d)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nDirection==DSL_NEAR_END ? "NE":"FE",nChannel));

   /* Clear the output structure*/
   memset(pCounters, 0x0, sizeof(DSL_PM_DataPathData_t));

   /* not for ADSL-only mode */
   if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2))
   {
      if( nDirection == DSL_NEAR_END )
      {
         memset(&(sAck.ackG997PtmStatsGet), 0x0, sizeof(sAck.ackG997PtmStatsGet));

         sCmd.cmdG997PtmStatsGet.Index  = 0x0;
         sCmd.cmdG997PtmStatsGet.Length = 8;
         sCmd.cmdG997PtmStatsGet.LinkNo = 0;

         /* Get PTM statistics*/
         nErrCode = DSL_DRV_VNX_SendMessage(
            pContext, CMD_G997_PTM_STATSGET,
            sizeof(sCmd.cmdG997PtmStatsGet), (DSL_uint8_t*)&(sCmd.cmdG997PtmStatsGet),
            sizeof(sAck.ackG997PtmStatsGet), (DSL_uint8_t*)&(sAck.ackG997PtmStatsGet));

         /* Check nErrCode and return on error*/
         if( nErrCode < 0 )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - PTM statistics get failed!" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            return nErrCode;
         }

         /* Fill the output data structure*/
         pCounters->nCRC_P  = sAck.ackG997PtmStatsGet.cntCRC_NE;
         pCounters->nCV_P   = sAck.ackG997PtmStatsGet.cntCVC_NE;
      }
      else
      {
         DSL_DEBUG( DSL_DBG_WRN,
            (pContext, "DSL[%02d]: WARNING - Data Path counters are not supported for the VDSL FE!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
      }
   }
   else if (DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) ||
            DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB))
   {
   /*... not for VDSL-only mode */
      if( nDirection == DSL_NEAR_END )
      {
        memset(&(sAck.ackG997AtmStatsGet), 0x0, sizeof(sAck.ackG997AtmStatsGet));

         sCmd.cmdG997AtmStatsGet.Index  = 0x0;
         sCmd.cmdG997AtmStatsGet.Length = 16;
         sCmd.cmdG997AtmStatsGet.LinkNo = 0x0;

         /* Get ATM statistics*/
         nErrCode = DSL_DRV_VNX_SendMessage(
            pContext, CMD_G997_ATM_STATSGET,
            sizeof(sCmd.cmdG997AtmStatsGet), (DSL_uint8_t*)&(sCmd.cmdG997AtmStatsGet),
            sizeof(sAck.ackG997AtmStatsGet), (DSL_uint8_t*)&(sAck.ackG997AtmStatsGet));

         /* Check nErrCode and return on error*/
         if( nErrCode < 0 )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - ATM statistics get failed!" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
               
            return nErrCode;
         }

         /* Fill the output data structure*/
         pCounters->nHEC            = sAck.ackG997AtmStatsGet.hecp;
         pCounters->nTotalCells     = sAck.ackG997AtmStatsGet.cdp;
         pCounters->nUserTotalCells = sAck.ackG997AtmStatsGet.cup;
         pCounters->nIBE            = sAck.ackG997AtmStatsGet.ibep;
      }
      else
      {
         DSL_DEBUG( DSL_DBG_WRN,
            (pContext, "DSL[%02d]: WARNING - Data Path Counters are not supported "
            "for the ADSL FE!" DSL_DRV_CRLF,
            DSL_DEV_NUM(pContext)));
      }
   }
   else
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Unknown operation mode!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      nErrCode = DSL_ERROR;
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_DEV_DataPathCountersCurrentGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_DataPathCountersSet(
   DSL_Context_t *pContext,
   DSL_uint8_t nChannel,
   DSL_XTUDir_t nDirection,
   DSL_PM_DataPathData_t *pCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_LineStateValue_t nLineState = DSL_LINESTATE_UNKNOWN;
   union
   {
      CMD_G997_PTM_StatsSet_t   PTM;
      CMD_G997_ATM_StatsSet_t   ATM;
   } sCmd;

   union
   {
      ACK_G997_PTM_StatsSet_t   PTM;
      ACK_G997_ATM_StatsSet_t   ATM;
   } sAck;


   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DEV_DataPathCountersSet, (nDirection=%s, nChannel=%d)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nDirection==DSL_NEAR_END ? "NE":"FE",nChannel));

   /* Restore counters only if the last showtime FW mode equals to the
      current loaded FW binary*/
   if( ((DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) ||
         DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1)) &&
        (DSL_DRV_PM_CONTEXT(pContext)->nLastShowtime == DSL_PM_FWMODE_VDSL)) ||
       ((DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) ||
         DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB)) &&
        (DSL_DRV_PM_CONTEXT(pContext)->nLastShowtime == DSL_PM_FWMODE_ADSL)) )
   {
      /* Check line state*/
      if( DSL_DRV_DEV_LineStateGet(pContext, &nLineState) != DSL_ERROR )
      {
         if( nLineState != DSL_LINESTATE_IDLE )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - PM data path counters restore only allowed "
               "in the IDLE state"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
            return DSL_ERROR;
         }
      }

      if( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
      {
         memset(&(sCmd.PTM), 0x0, sizeof(sCmd.PTM));
         sCmd.PTM.Length = 9;
         sCmd.PTM.LinkNo = 0;

         sCmd.PTM.cntCRC_NE = pCounters->nCRC_P;
         sCmd.PTM.cntCVC_NE = pCounters->nCV_P;

         /* Set PTM statistics*/
         nErrCode = DSL_DRV_VNX_SendMessage(
            pContext, CMD_G997_PTM_STATSSET,
            sizeof(sCmd.PTM), (DSL_uint8_t*)&(sCmd.PTM),
            sizeof(sAck.PTM), (DSL_uint8_t*)&(sAck.PTM));

         if( nErrCode != DSL_SUCCESS )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - ATM statistics Set failed!" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
            
            return nErrCode;
         }
      }
      else
      {
         memset(&(sCmd.ATM), 0x0, sizeof(sCmd.ATM));
         sCmd.ATM.Length = DSL_VNX_32BIT_WR_MSG_LEN_GET(sCmd.ATM);
         sCmd.ATM.LinkNo = 0;

         /* Fill Near-end Hec Violation Count (HEC-P), Current Period*/
         sCmd.ATM.hecp = pCounters->nHEC;
         /* Fill Near-end Delineated Total Cell Count (CD-P), Current Period*/
         sCmd.ATM.cdp  = pCounters->nTotalCells;
         /* Fill Near-end User Total Cell Count (CU-P), Current Period*/
         sCmd.ATM.cup  = pCounters->nUserTotalCells;
         /* Fill Near-end Idle Cell Bit Error Count (IBE-P), Current Period*/
         sCmd.ATM.ibep = pCounters->nIBE;

         /* Set ATM statistics*/
         nErrCode = DSL_DRV_VNX_SendMessage(
            pContext, CMD_G997_ATM_STATSSET,
            sizeof(sCmd.ATM), (DSL_uint8_t*)&(sCmd.ATM),
            sizeof(sAck.ATM), (DSL_uint8_t*)&(sAck.ATM));

         if( nErrCode != DSL_SUCCESS )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - ATM statistics Set failed!" DSL_DRV_CRLF,
               DSL_DEV_NUM(pContext)));
            
            return nErrCode;
         }
      }
   }
   else
   {
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: FW mode switch detected, PM data path counters restore skipped..."
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_DEV_DataPathCountersSet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}
#endif /* #ifdef INCLUDE_DSL_CPE_PM_DATA_PATH_COUNTERS*/

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_LineSecCountersGet(
   DSL_Context_t *pContext,
   DSL_XTUDir_t nDirection,
   DSL_PM_LineSecData_t *pCounters)
{
   DSL_Error_t    nErrCode = DSL_SUCCESS;

   union
   {
      CMD_LinePerfCountNE_Get_t      VDSL_NE;
      CMD_LinePerfCountFE_Get_t      VDSL_FE;
      CMD_LinePerfCountNE_NoTR1Get_t ADSL_NE;
   } sCmd;

   union
   {
      ACK_LinePerfCountNE_Get_t      VDSL_NE;
      ACK_LinePerfCountFE_Get_t      VDSL_FE;
      ACK_LinePerfCountNE_NoTR1Get_t ADSL_NE;
   } sAck;


   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_ATU_DIRECTION(nDirection);
   DSL_CHECK_ERR_CODE();

   /* Reset the output structure*/
   memset(pCounters, 0x0, sizeof(DSL_PM_LineSecData_t));

   /* not for ADSL, Far-End mode */
   if( !DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) && nDirection == DSL_FAR_END )
   {
      DSL_DEBUG( DSL_DBG_WRN,
         (pContext, "DSL[%02d]: WARNING - Line Sec counters are not supported for the FE!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return DSL_SUCCESS;
    }

    DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_PM_DEV_LineSecCountersCurrentGet" DSL_DRV_CRLF,
       DSL_DEV_NUM(pContext)));

   memset(&(sCmd), 0, sizeof(sCmd));
   memset(&(sAck), 0, sizeof(sAck));

   /* For VDSL only mode*/
   if( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
   {
      sCmd.VDSL_NE.Length = 20;

      nErrCode = DSL_DRV_VNX_SendMessage(
         pContext,
         nDirection == DSL_NEAR_END ? CMD_LINEPERFCOUNTNE_GET : CMD_LINEPERFCOUNTFE_GET,
         nDirection == DSL_NEAR_END ? sizeof(sCmd.VDSL_NE) :
                                      sizeof(sCmd.VDSL_FE),
         nDirection == DSL_NEAR_END ? (DSL_uint8_t*)&(sCmd.VDSL_NE) :
                                      (DSL_uint8_t*)&(sCmd.VDSL_FE),
         nDirection == DSL_NEAR_END ? sizeof(sAck.VDSL_NE) :
                                      sizeof(sAck.VDSL_FE),
         nDirection == DSL_NEAR_END ? (DSL_uint8_t*)&(sAck.VDSL_NE) :
                                      (DSL_uint8_t*)&(sAck.VDSL_FE));

      if( nErrCode == DSL_SUCCESS )
      {
         if( nDirection == DSL_NEAR_END )
         {
            /* Get current ES value*/
            pCounters->nES   = (sAck.VDSL_NE.cntES_NE_MSW << 16) | sAck.VDSL_NE.cntES_NE_LSW;
            /* Get current SES value*/
            pCounters->nSES  = (sAck.VDSL_NE.cntSES_NE_MSW << 16) | sAck.VDSL_NE.cntSES_NE_LSW;
            /* Get current LOSS value*/
            pCounters->nLOSS  = (sAck.VDSL_NE.cntLOSS_NE_MSW << 16) | sAck.VDSL_NE.cntLOSS_NE_LSW;
            /* Get current UAS value*/
            pCounters->nUAS  = (sAck.VDSL_NE.cntUAS_NE_MSW << 16) | sAck.VDSL_NE.cntUAS_NE_LSW;
            /* Not supported by the VINAX VDSL FW*/
            pCounters->nLOFS = 0;
         }
         else
         {
            /* Get current ES value*/
            pCounters->nES   = (sAck.VDSL_FE.cntES_FE_MSW << 16) | sAck.VDSL_FE.cntES_FE_LSW;
            /* Get current SES value*/
            pCounters->nSES  = (sAck.VDSL_FE.cntSES_FE_MSW << 16) | sAck.VDSL_FE.cntSES_FE_LSW;
            /* Get current LOSS value*/
            pCounters->nLOSS  = (sAck.VDSL_FE.cntLOSS_FE_MSW << 16) | sAck.VDSL_FE.cntLOSS_FE_LSW;
            /* Get current UAS value*/
            pCounters->nUAS  = (sAck.VDSL_FE.cntUAS_FE_MSW << 16) | sAck.VDSL_FE.cntUAS_FE_LSW;
            /* Not supported by the VINAX VDSL FW*/
            pCounters->nLOFS = 0;
         }
      }
   }
   else if(DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) ||
           DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB))
   {
      sCmd.ADSL_NE.Length = 10;

      nErrCode = DSL_DRV_VNX_SendMessage(
         pContext,
         CMD_LINEPERFCOUNTNE_NOTR1GET,
         sizeof(sCmd.ADSL_NE), (DSL_uint8_t*)&(sCmd.ADSL_NE),
         sizeof(sAck.ADSL_NE), (DSL_uint8_t*)&(sAck.ADSL_NE));

      if( nErrCode == DSL_SUCCESS )
      {
         /* Get current ES value*/
         pCounters->nES   = (sAck.ADSL_NE.cntES_MSW << 16) | sAck.ADSL_NE.cntES_LSW;
         /* Get current SES value*/
         pCounters->nSES  = (sAck.ADSL_NE.cntSES_MSW << 16) | sAck.ADSL_NE.cntSES_LSW;
         /* Get current LOSS value*/
         pCounters->nLOSS = (sAck.ADSL_NE.cntLOSS_MSW << 16) | sAck.ADSL_NE.cntLOSS_LSW;
         /* UAS counter not supported by the FW in the ADSL mode*/
         pCounters->nUAS  = 0;
         /* LOFS counter not supported by the FW in the ADSL mode*/
         pCounters->nLOFS = 0;
      }
   }
   else
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Unknown operation mode!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
           
      nErrCode = DSL_ERROR;
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_DEV_LineSecCountersCurrentGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_LineSecCountersSet(
   DSL_Context_t *pContext,
   DSL_XTUDir_t nDirection,
   DSL_PM_LineSecData_t *pCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_LineStateValue_t nLineState = DSL_LINESTATE_UNKNOWN;

   union
   {
      CMD_LinePerfCountNE_NoTR1Set_t LinePerfNEadsl;
      /* Near-End and Far-End structures are the same*/
      CMD_LinePerfCountNE_Set_t      LinePerfNEvdsl;
      CMD_LinePerfCountFE_Set_t      LinePerfFEvdsl;
   } sCmd;

   union
   {
      ACK_LinePerfCountNE_NoTR1Set_t LinePerfNEadsl;
      /* Near-End and Far-End structures are the same*/
      ACK_LinePerfCountNE_Set_t      LinePerfNEvdsl;
      ACK_LinePerfCountFE_Set_t      LinePerfFEvdsl;
   } sAck;


   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DEV_LineSecCountersSet, (nDirection=%s)" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nDirection==DSL_NEAR_END ? "NE":"FE"));

   /* Restore counters only if the last showtime FW mode equals to the
      current loaded FW binary*/
   if( ((DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) ||
         DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL1)) &&
        (DSL_DRV_PM_CONTEXT(pContext)->nLastShowtime == DSL_PM_FWMODE_VDSL)) ||
       ((DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLA) ||
         DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_ADSLB)) &&
        (DSL_DRV_PM_CONTEXT(pContext)->nLastShowtime == DSL_PM_FWMODE_ADSL)) )
   {
      /* Check line state*/
      if( DSL_DRV_DEV_LineStateGet(pContext, &nLineState) != DSL_ERROR )
      {
         if( nLineState != DSL_LINESTATE_IDLE )
         {
            DSL_DEBUG( DSL_DBG_ERR,
               (pContext, "DSL[%02d]: ERROR - PM line sec counters restore only allowed "
               "in the IDLE state"DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
            
            return DSL_ERROR;
         }
      }

      /* Only applicable for ADSL mode*/
      if( DSL_DRV_VNX_FwFeatureCheck(pContext, DSL_VNX_FW_VDSL2) )
      {
         memset(&(sCmd.LinePerfNEvdsl), 0x0, sizeof(sCmd.LinePerfNEvdsl));

         /* Fill the message*/
         sCmd.LinePerfNEvdsl.Length         = 20;
         sCmd.LinePerfNEvdsl.cntES_NE_MSW   = (DSL_uint16_t)(pCounters->nES>>16);
         sCmd.LinePerfNEvdsl.cntES_NE_LSW   = (DSL_uint16_t)pCounters->nES;
         sCmd.LinePerfNEvdsl.cntSES_NE_MSW  = (DSL_uint16_t)(pCounters->nSES>>16);
         sCmd.LinePerfNEvdsl.cntSES_NE_LSW  = (DSL_uint16_t)(pCounters->nSES);
         sCmd.LinePerfNEvdsl.cntES_NE_MSW   = (DSL_uint16_t)(pCounters->nLOSS>>16);
         sCmd.LinePerfNEvdsl.cntLOSS_NE_LSW = (DSL_uint16_t)(pCounters->nLOSS);
         sCmd.LinePerfNEvdsl.cntUAS_NE_MSW  = (DSL_uint16_t)(pCounters->nUAS>>16);
         sCmd.LinePerfNEvdsl.cntUAS_NE_LSW  = (DSL_uint16_t)(pCounters->nUAS);

         /* Set Line sec counters*/
         nErrCode =  DSL_DRV_VNX_SendMessage(
            pContext, 
            nDirection == DSL_NEAR_END ? CMD_LINEPERFCOUNTNE_SET : CMD_LINEPERFCOUNTFE_SET,
            sizeof(sCmd.LinePerfNEvdsl), (DSL_uint8_t *)&(sCmd.LinePerfNEvdsl),
            sizeof(sAck.LinePerfNEvdsl), (DSL_uint8_t *)&(sAck.LinePerfNEvdsl));
      }
      else
      {
         if (nDirection == DSL_NEAR_END)
         {
            memset(&(sCmd.LinePerfNEadsl), 0x0, sizeof(sCmd.LinePerfNEadsl));

            /* Fill the message*/
            sCmd.LinePerfNEadsl.Length      = DSL_VNX_16BIT_RD_MSG_LEN_GET(sCmd);
            sCmd.LinePerfNEadsl.cntES_MSW   = (DSL_uint16_t)(pCounters->nES>>16);
            sCmd.LinePerfNEadsl.cntES_LSW   = (DSL_uint16_t)pCounters->nES;
            sCmd.LinePerfNEadsl.cntSES_MSW  = (DSL_uint16_t)(pCounters->nSES>>16);
            sCmd.LinePerfNEadsl.cntSES_LSW  = (DSL_uint16_t)(pCounters->nSES);
            sCmd.LinePerfNEadsl.cntLOSS_MSW = (DSL_uint16_t)(pCounters->nLOSS>>16);
            sCmd.LinePerfNEadsl.cntLOSS_LSW = (DSL_uint16_t)(pCounters->nLOSS);
            sCmd.LinePerfNEadsl.cntUAS_MSW  = (DSL_uint16_t)(pCounters->nUAS>>16);
            sCmd.LinePerfNEadsl.cntUAS_LSW  = (DSL_uint16_t)(pCounters->nUAS);

            /* Set Line sec counters*/
            nErrCode =  DSL_DRV_VNX_SendMessage(
               pContext, CMD_LINEPERFCOUNTNE_NOTR1SET,
               sizeof(sCmd), (DSL_uint8_t *)&sCmd,
               sizeof(sAck), (DSL_uint8_t *)&sAck);
         }
      }
   }
   else
   {
      DSL_DEBUG( DSL_DBG_MSG,
         (pContext, "DSL[%02d]: FW mode switch detected, PM line sec counters restore skipped..."
         DSL_DRV_CRLF, DSL_DEV_NUM(pContext)));
   }

   if( nErrCode != DSL_SUCCESS )
   {
      DSL_DEBUG( DSL_DBG_ERR,
         (pContext, "DSL[%02d]: ERROR - Line sec Counters Set failed!" DSL_DRV_CRLF,
         DSL_DEV_NUM(pContext)));
         
      return nErrCode;
   }

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_DEV_LineSecCountersSet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

#ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS
/*
   For a detailed description of the function, its arguments and return value
   please refer to the description in the header file 'drv_dsl_cpe_device_pm.h'
*/
DSL_Error_t DSL_DRV_PM_DEV_LineInitCountersGet(
   DSL_Context_t *pContext,
   DSL_PM_LineInitData_t *pCounters)
{
   DSL_Error_t nErrCode = DSL_SUCCESS;
   DSL_uint32_t nFullInits = 0, nFailedFullInits = 0;

   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DEV_LineInitCountersCurrentGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Reset the output structure*/
   memset(pCounters, 0x0, sizeof(DSL_PM_LineInitData_t));

   /* Get Line initialization performance counters*/
   DSL_CTX_READ(pContext, nErrCode, nFullInits, nFullInits);
   DSL_CTX_READ(pContext, nErrCode, nFailedFullInits, nFailedFullInits);

   pCounters->nFullInits        = nFullInits;
   pCounters->nFailedFullInits  = nFailedFullInits;
   /* Short Init stuff is not supported*/
   pCounters->nShortInits       = 0;
   pCounters->nFailedShortInits = 0;

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: OUT - DSL_DRV_PM_DEV_LineInitCountersCurrentGet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   return nErrCode;
}

DSL_Error_t DSL_DRV_PM_DEV_LineInitCountersSet(
   DSL_Context_t *pContext,
   DSL_PM_LineInitData_t *pCounters)
{
   DSL_Error_t    nErrCode = DSL_SUCCESS;

   DSL_CHECK_POINTER(pContext, pCounters);
   DSL_CHECK_ERR_CODE();

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DEV_LineInitCountersSet" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext)));

   /* Today, Line Init counters are completely handled on the API level*/

   DSL_DEBUG( DSL_DBG_MSG,
      (pContext, "DSL[%02d]: IN - DSL_DRV_PM_DEV_LineInitCountersSet, retCode" DSL_DRV_CRLF,
      DSL_DEV_NUM(pContext), nErrCode));

   return nErrCode;
}
#endif /* #ifdef INCLUDE_DSL_CPE_PM_LINE_COUNTERS*/

#endif /* #ifdef defined(INCLUDE_DSL_CPE_API_VINAX) && defined(INCLUDE_DSL_PM) */
