#ifndef __DANUBE_PPA_API_H__20060913_1556__
#define __DANUBE_PPA_API_H__20060913_1556__



/******************************************************************************
**
** FILE NAME    : danube_ppa_api.h
** PROJECT      : Twinpass-E
** MODULES      : PPA (Routing Acceleration)
**
** DATE         : 13 SEP 2006
** AUTHOR       : Xu Liang
** DESCRIPTION  : PPA (Routing Acceleration) Protocol Stack Hook API Header File
** COPYRIGHT    :   Copyright (c) 2006
**          Infineon Technologies AG
**          Am Campeon 1-12, 85579 Neubiberg, Germany
**
**   Any use of this software is subject to the conclusion of a respective
**   License agreement. Without such a License agreement no rights to the
**   software are granted
**
** HISTORY
** $Date        $Author         $Comment
** 13 SEP 2006  Xu Liang        Initiate Version
** 26 OCT 2006  Xu Liang        Add GPL header.
*******************************************************************************/



#if defined(CONFIG_IFX_PPA_A4)
  #include <asm/amazon_se/amazon_se_ppa_ppe_a4_hal.h>
  #include <asm/amazon_se/ifx_ppa_stack_al.h>
#endif



/*
 * ####################################
 *              Definition
 * ####################################
 */

#define USE_NEW_MC_API                          1

#define MAX_MC_IFS_NUM                          8
#define PPA_MAX_DEV				8
//#define PPA_MAX_DIRECTPATH_DEV		3

#define IFX_TRUE				1
#define IFX_FALSE				0

#define IFX_SUCCESS				1
#define IFX_FAILURE				0

/*
 *  ioctl Command
 */
#define PPA_IOC_MAGIC                           ((uint32_t)'p')
#define PPA_CMD_INIT                            _IOW(PPA_IOC_MAGIC, 0, PPA_CMD_INIT_INFO)
#define PPA_CMD_EXIT                            _IO(PPA_IOC_MAGIC, 1)
#define PPA_CMD_ENABLE                          _IOW(PPA_IOC_MAGIC, 2, PPA_CMD_ENABLE_INFO)
#define PPA_CMD_GET_STATUS                      _IOR(PPA_IOC_MAGIC, 3, PPA_CMD_GET_STATUS_INFO)
#define PPA_CMD_SET_IF_MAC                      _IOW(PPA_IOC_MAGIC, 4, PPA_CMD_IF_MAC_INFO)
#define PPA_CMD_GET_IF_MAC                      _IOWR(PPA_IOC_MAGIC, 5, PPA_CMD_IF_MAC_INFO)
#define PPA_CMD_SET_BR_VLAN                     _IOW(PPA_IOC_MAGIC, 6, PPA_CMD_BR_IF_VLAN_CFG)
#define PPA_CMD_ADD_BR_ENTRY                    _IOW(PPA_IOC_MAGIC, 7, PPA_IOCTL_VLAN_MAP_ENTRY)
#define PPA_CMD_DEL_BR_ENTRY                    _IOW(PPA_IOC_MAGIC, 8, PPA_IOCTL_VLAN_MAP_ENTRY)
#define PPA_CMD_ERASE_ALL_BR_ENTRY              _IOW(PPA_IOC_MAGIC, 9, u32)
#define PPA_CMD_SET_BR_VLAN_PRIO                _IOW(PPA_IOC_MAGIC, 10, PPA_IOCTL_BR_VLAN_PRIO)
#define PPA_CMD_ENABLE_DIRECTPATH		_IOW(PPA_IOC_MAGIC, 11, u32)
#define PPA_CMD_DISABLE_DIRECTPATH		_IOW(PPA_IOC_MAGIC, 12, u32)
#define PPA_IOC_MAXNR                           13

/*
 *  internal flag
 */

#define SESSION_INTERNAL_FLAG_BASE              0
#define SESSION_IS_REPLY                        0x00000001
#define SESSION_BRIDGING_VCI_CHECK              0x00000002
#define SESSION_ADDED_IN_HW                     0x00000010
#define SESSION_CAN_NOT_ACCEL                   0x00000020
#define SESSION_STATIC                          0x00000040
#define SESSION_DROP                            0x00000080
#define SESSION_VALID_NAT_IP                    0x00000100
#define SESSION_VALID_NAT_PORT                  0x00000200
#define SESSION_VALID_NAT_SNAT                  0x00000400  //  src IP is replaced, otherwise dest IP is replaced
#define SESSION_VALID_VLAN_INS                  0x00001000
#define SESSION_VALID_VLAN_RM                   0x00002000
#define SESSION_VALID_OUT_VLAN_INS              0x00004000
#define SESSION_VALID_OUT_VLAN_RM               0x00008000
#define SESSION_VALID_PPPOE                     0x00010000
#define SESSION_VALID_NEW_SRC_MAC               0x00020000
#define SESSION_VALID_SRC_MAC                   SESSION_VALID_NEW_SRC_MAC
#define SESSION_VALID_MTU                       0x00040000
#define SESSION_VALID_NEW_DSCP                  0x00080000
#define SESSION_LAN_ENTRY                       0x00100000
#define SESSION_WAN_ENTRY                       0x00200000
#define SESSION_SRC_MAC_DROP_EN                 0x01000000
#define SESSION_TX_ITF_IPOA                     0x02000000
#define SESSION_TX_ITF_PPPOA                    0x04000000
#define SESSION_TX_ITF_IPOA_PPPOA_MASK          (SESSION_TX_ITF_IPOA | SESSION_TX_ITF_PPPOA)


#define PPA_IS_NAT_SESSION(flags)               ((flags) & (SESSION_VALID_NAT_IP | SESSION_VALID_NAT_PORT))

/*
 *  flags
 */
#define PPA_F_BEFORE_NAT_TRANSFORM              0x00000001
#define PPA_F_SESSION_ORG_DIR                   0x00000010
#define PPA_F_SESSION_REPLY_DIR                 0x00000020
#define PPA_F_SESSION_BIDIRECTIONAL             (PPA_F_SESSION_ORG_DIR | PPA_F_SESSION_REPLY_DIR)
#define PPA_F_BRIDGED_SESSION                   0x00000100
#define PPA_F_SESSION_NEW_DSCP                  0x00001000
#define PPA_F_SESSION_VLAN                      0x00002000
#define PPA_F_MTU                               0x00004000
#define PPA_F_SESSION_OUT_VLAN                  0x00008000
#define PPA_F_BRIDGE_LOCAL                      0x00010000
#define PPA_F_DIRECTPATH_DEREGISTER             0x00100000
#define PPA_F_STATIC_ENTRY                      0x20000000
#define PPA_F_DROP_PACKET                       0x40000000
#define PPA_F_BRIDGE_ACCEL_MODE                 0x80000000
#define PPA_F_DIRECTPATH_ETH_IF			0x01000000
// TODO: value ??
#define PPA_F_DIRECTPATH_CORE1		        0x10000000	
/*
 *  interface flags
 */
#define PPA_F_IF_ALLOW_BROADCAST_MULTICAST      0x00000001

/*
 *  ifx_ppa_session_add return value
 */
#define IFX_PPA_SESSION_NOT_ADDED               -1
#define IFX_PPA_SESSION_ADDED                   0
#define IFX_PPA_SESSION_EXISTS                  1

/*
 *  ifx_ppa_inactivity_status return value
 */
//#define IFX_PPA_SESSION_NOT_ADDED               -1
#define IFX_PPA_HIT                             0
#define IFX_PPA_TIMEOUT                         1

/*
 *  interface id
 */
#define IFX_PPA_IF_ID_ETH0                      0
#define IFX_PPA_IF_ID_ETH1                      1
#define IFX_PPA_IF_ID_ATM                       IFX_PPA_IF_ID_ETH1
#define IFX_PPA_IF_ID_CPU0                      2
#define IFX_PPA_IF_ID_CPU0_EXT_IF0              3
#define IFX_PPA_IF_ID_CPU0_EXT_IF1              4
#define IFX_PPA_IF_ID_CPU1_EXT_IF0              5
#define IFX_PPA_IF_ID_CPU1_EXT_IF1              6
#define IFX_PPA_IF_ID_CPU1_EXT_IF2              7

/*
 *  bridging VLAN criteria type
 */
#define PPA_F_VLAN_FILTER_IFNAME                0
#define PPA_F_VLAN_FILTER_IP_SRC                1
#define PPA_F_VLAN_FILTER_ETH_PROTO             2
#define PPA_F_VLAN_FILTER_VLAN_TAG              3


#define ENABLE_SESSION_DEBUG_FLAGS              1

/*
 * ####################################
 *              Data Type
 * ####################################
 */
typedef struct {
    uint32_t    new_dscp            :6;
    uint32_t    dscp_remark         :1;
    uint32_t    vlan_insert         :1;
    uint32_t    vlan_remove         :1;
    uint32_t    out_vlan_insert     :1;     //  valid in A4
    uint32_t    out_vlan_remove     :1;     //  valid in A4
    uint32_t    reserved1           :5;
    uint32_t    vlan_prio           :3;
    uint32_t    vlan_cfi            :1;
    uint32_t    vlan_id             :12;
    uint16_t    mtu;
    uint32_t    session_flags;
    uint32_t    out_vlan_tag;               //  valid in A4
} PPA_SESSION_EXTRA;


struct session_list_item {
    struct session_list_item   *next;
//    uint32_t                    use_count;

    PPA_SESSION                *session;
    PPA_SESSION_EXTRA           session_extra;
    uint16_t                    ip_proto;
    uint32_t                    src_ip;
    uint16_t                    src_port;
    uint8_t                     src_mac[PPA_ETH_ALEN];
    uint32_t                    dst_ip;
    uint16_t                    dst_port;
    uint8_t                     dst_mac[PPA_ETH_ALEN];
    uint32_t                    nat_ip;         //  IP address to be replaced by NAT if NAT applies
    uint16_t                    nat_port;       //  Port to be replaced by NAT if NAT applies
//    uint8_t                     new_src_mac[PPA_ETH_ALEN];  //  src MAC address to be replaced
    PPA_NETIF                  *rx_if;
    PPA_NETIF                  *tx_if;
    uint32_t                    timeout;
    uint32_t                    last_hit_time;  //  Updated by bookkeeping thread
    uint32_t                    num_adds;       //  Number of attempts to add session
    uint32_t                    pppoe_session_id;
    uint32_t                    new_dscp;
    uint32_t                    new_vci;
    uint32_t                    mtu;
#if !defined(USE_NEW_MC_API) || !USE_NEW_MC_API
    uint32_t                    mc_dest_list;
#endif
    uint32_t                    flags;          //  Internal flag : SESSION_IS_REPLY,
                                                //                  SESSION_ADDED_IN_HW, SESSION_CAN_NOT_ACCEL
                                                //                  SESSION_VALID_NAT_IP, SESSION_VALID_NAT_PORT,
                                                //                  SESSION_VALID_VLAN_INS, SESSION_VALID_VLAN_RM,
                                                //                  SESSION_VALID_OUT_VLAN_INS, SESSION_VALID_OUT_VLAN_RM,
                                                //                  SESSION_VALID_PPPOE, SESSION_VALID_SRC_MAC,
                                                //                  SESSION_VALID_MTU,
                                                //                  SESSION_VALID_NEW_DSCP,
                                                //                  SESSION_LAN_ENTRY, SESSION_WAN_ENTRY

    uint32_t                    routing_entry;
#if !defined(USE_NEW_MC_API) || !USE_NEW_MC_API
    uint32_t                    mc_routing_entry;
#endif
    uint32_t                    pppoe_entry;
    uint32_t                    mtu_entry;
    uint32_t                    src_mac_entry;

#ifdef CONFIG_IFX_PPA_A4
    uint32_t                    dslwan_qid;
    uint32_t                    out_vlan_tag;
    uint32_t                    out_vlan_ix;
#endif
#if defined(ENABLE_SESSION_DEBUG_FLAGS) && ENABLE_SESSION_DEBUG_FLAGS
    uint32_t                    debug_flags;
#endif
};

#if defined(USE_NEW_MC_API) && USE_NEW_MC_API
  struct mc_group_list_item {
    struct mc_group_list_item  *next;

    IPADDR                      ip_mc_group;
    uint32_t                    num_ifs;
    PPA_NETIF                  *netif[MAX_MC_IFS_NUM];
    uint32_t                    ttl[MAX_MC_IFS_NUM];
    uint32_t                    if_mask;
    uint32_t                    new_vci;
    uint32_t                    flags;          //  Internal flag : SESSION_IS_REPLY,
                                                //                  SESSION_ADDED_IN_HW, SESSION_CAN_NOT_ACCEL
                                                //                  SESSION_VALID_NAT_IP, SESSION_VALID_NAT_PORT,
                                                //                  SESSION_VALID_VLAN_INS, SESSION_VALID_VLAN_RM,
                                                //                  SESSION_VALID_OUT_VLAN_INS, SESSION_VALID_OUT_VLAN_RM,
                                                //                  SESSION_VALID_PPPOE, SESSION_VALID_SRC_MAC,
                                                //                  SESSION_VALID_MTU,
                                                //                  SESSION_VALID_NEW_DSCP,
                                                //                  SESSION_LAN_ENTRY, SESSION_WAN_ENTRY

    uint32_t                    mc_entry;
    uint32_t                    src_mac_entry;

#ifdef CONFIG_IFX_PPA_A4
    uint32_t                    out_vlan_tag;
    uint32_t                    out_vlan_ix;
#endif
#if defined(ENABLE_SESSION_DEBUG_FLAGS) && ENABLE_SESSION_DEBUG_FLAGS
    uint32_t                    debug_flags;
#endif
  };
#endif

struct bridging_session_list_item {
    struct bridging_session_list_item *next;
//    uint32_t                    use_count;

    uint8_t                     mac[PPA_ETH_ALEN];
    uint32_t                    vci;
    PPA_NETIF                  *netif;
    uint32_t                    new_vci;
    uint32_t                    timeout;
    uint32_t                    last_hit_time;  //  Updated by bookkeeping thread
    uint32_t                    flags;          //  Internal flag : SESSION_BRIDGING_VCI_CHECK
                                                //                  SESSION_ADDED_IN_HW, SESSION_CAN_NOT_ACCEL
                                                //                  SESSION_STATIC, SESSION_DROP
                                                //                  SESSION_VALID_VLAN_INS, SESSION_VALID_VLAN_RM
                                                //                  SESSION_SRC_MAC_DROP_EN
    uint32_t                    bridging_entry;

#ifdef CONFIG_IFX_PPA_A4
    uint32_t                    dslwan_qid;
#endif
#if defined(ENABLE_SESSION_DEBUG_FLAGS) && ENABLE_SESSION_DEBUG_FLAGS
    uint32_t                    debug_flags;
#endif
};



/*
 *  API structures
 */

typedef struct {
    PPA_IFNAME *ifname;
    uint32_t    if_flags;
} PPA_IFINFO;

typedef struct ppa_verify_checks {
    uint32_t    f_ip_verify             :1; //  Enable/Disable IP verification checks
    uint32_t    f_tcp_udp_verify        :1; //  Enable/Disable TCP/UDP verification checks
    uint32_t    f_tcp_udp_err_drop      :1; //  Enable/Disable drop packet if TCP/UDP checksum is wrong
    uint32_t    f_drop_on_no_hit        :1; //  Drop unicast packets on no hit, forward to MIPS CPU otherwise (default)
    uint32_t    f_mc_drop_on_no_hit     :1; //  Drop multicast on no hit, forward to MIPS CPU otherwise (default)
} PPA_VERIFY_CHECKS;

typedef struct {
    PPA_VERIFY_CHECKS   lan_rx_checks;      // LAN Ingress checks
    PPA_VERIFY_CHECKS   wan_rx_checks;      // WAN Ingress checks
    uint32_t    num_lanifs;                 //  Number of LAN side interfaces
    PPA_IFINFO *p_lanifs;
    uint32_t    num_wanifs;                 //  Number of WAN side interfaces
    PPA_IFINFO *p_wanifs;
    uint32_t    max_lan_source_entries;     //  Number of session entries with LAN source
    uint32_t    max_wan_source_entries;     //  Number of session entries with WAN source
    uint32_t    max_mc_entries;             //  Number of multicast sessions
    uint32_t    max_bridging_entries;       //  Number of bridging entries

    uint32_t    add_requires_min_hits;      // Minimum number of calls to ppa_add before session would be added in h/w
} PPA_INIT_INFO;

#if !defined(USE_NEW_MC_API) || !USE_NEW_MC_API
  typedef struct {
    uint32_t    ip_mc_group;            //  IP multicast group address
    int8_t      num_ifs;                //  Number of output interfaces
//    PPA_IFNAME **array_mem_ifs;         //  Array of output interfaces' names
    PPA_IFINFO *array_mem_ifs;
    uint8_t     replace_src_mac;        //  Whether to replace source MAC address
    uint8_t     src_mac[PPA_ETH_ALEN];  //  Source MAC address to replace
  } PPA_MC_SESSION;
#else
  typedef struct {
    PPA_IFNAME *ifname;
    uint8_t     ttl;
  } IF_TTL_ENTRY;

  typedef struct {
    IPADDR          ip_mc_group;
    int8_t          num_ifs;
    IF_TTL_ENTRY    array_mem_ifs[MAX_MC_IFS_NUM];
    uint8_t         if_mask;
    uint32_t        vlan_insert         :1;     //  valid in A4
    uint32_t        vlan_remove         :1;     //  valid in A4
    uint32_t        out_vlan_insert     :1;     //  valid in A4
    uint32_t        out_vlan_remove     :1;     //  valid in A4
    uint32_t        reserved1           :4;     //  valid in A4
    uint32_t        vlan_prio           :3;     //  valid in A4
    uint32_t        vlan_cfi            :1;     //  valid in A4
    uint32_t        vlan_id             :12;    //  valid in A4
    uint32_t        out_vlan_tag;
  } PPA_MC_GROUP;
#endif

typedef void PPA_U_SESSION;

typedef struct {
    uint32_t    tx_pkts;
    uint32_t    rx_pkts;
    uint32_t    tx_discard_pkts;
    uint32_t    tx_error_pkts;
    uint32_t    rx_discard_pkts;
    uint32_t    rx_error_pkts;
    uint32_t    tx_bytes;
    uint32_t    rx_bytes;
} PPA_IF_STATS;

typedef struct {
    uint32_t    fast_tcp_pkt_tx;
    uint32_t    fast_udp_pkt_tx;
    uint32_t    reserved1;
    uint32_t    fast_udp_mcast_pkt_tx;
    uint32_t    fast_tcp_pkt_drop;
    uint32_t    fast_udp_pkt_drop;
    uint32_t    reserved2;
    uint32_t    reserved3;
    uint32_t    rx_pkt_error;
} PPA_ACCEL_STATS;

#if 0
typedef struct {
    uint32_t    filter_type;
    uint32_t    ip_src,      ip_dst;
    uint32_t    ip_src_mask, ip_dst_mask;
    uint8_t     src_mac[PPA_ETH_ALEN], dst_mac[PPA_ETH_ALEN];
    uint32_t    spi;
    uint32_t    l4_protocol;
} PPA_FILTER;

typedef union {
    struct {
        uint32_t    udp_remove;
        uint32_t    process_xxx;
    } ipsec_action;
    struct {
        uint32_t    rate;
        uint32_t    rate_method;
        uint32_t    drop;
    } firwall_action;
} PPA_ACTION;
#endif

typedef struct {
    unsigned int    unmodified      :1;
    unsigned int    insertion       :1;
    unsigned int    remove          :1;
    unsigned int    replace         :1;
    unsigned int    out_unmodified  :1;
    unsigned int    out_insertion   :1;
    unsigned int    out_remove      :1;
    unsigned int    out_replace     :1;
} VLAN_TAG_CTRL;

typedef struct {
    unsigned int    src_ip_based_vlan   :1;
    unsigned int    eth_type_based_vlan :1;
    unsigned int    vlanid_based_vlan   :1;
    unsigned int    port_based_vlan     :1;
    unsigned int    vlan_aware          :1;
    unsigned int    out_vlan_aware      :1;
} VLAN_CFG;

typedef struct {
    union {
        PPA_IFNAME     *ifname;
        IPADDR          ip_src;
        //  to prevent big endian issue
        //uint16_t        eth_protocol;
        //uint16_t        ingress_vlan_tag;
        uint32_t        eth_protocol;
        uint32_t        ingress_vlan_tag;
    }           match_field;
    uint32_t    match_flags;
} PPA_VLAN_MATCH_FIELD;

typedef struct {
    uint16_t    vlan_vci;
    uint32_t    num_ifs;
    PPA_IFINFO *vlan_if_membership;
} PPA_VLAN_INFO;

typedef struct {
    PPA_VLAN_MATCH_FIELD    match_field;
    PPA_VLAN_INFO           vlan_info;
} PPA_VLAN_FILTER_CONFIG;

/*
 *  ioctl command structures
 */

typedef struct {
    PPA_IFNAME  ifname[PPA_IF_NAME_SIZE];
    uint32_t    if_flags;
} PPA_CMD_IFINFO;

typedef struct {
    PPA_INIT_INFO   init_info;  //  p_lanifs, p_wanifs is index of first PPA_CMD_IFINFO item
    uint32_t        flags;
    PPA_CMD_IFINFO  ifinfo[20];
} PPA_CMD_INIT_INFO;

typedef struct {
    uint32_t        lan_rx_ppa_enable;
    uint32_t        wan_rx_ppa_enable;
    uint32_t        flags;
} PPA_CMD_ENABLE_INFO;

typedef struct {
    uint32_t        lan_rx_ppa_enable;
    uint32_t        wan_rx_ppa_enable;
} PPA_CMD_GET_STATUS_INFO;

typedef struct {
    PPA_IFNAME  ifname[PPA_IF_NAME_SIZE];
    uint8_t     mac[PPA_ETH_ALEN];
} PPA_CMD_IF_MAC_INFO;

typedef struct
{
    PPA_IFNAME      if_name[PPA_IF_NAME_SIZE];
    VLAN_TAG_CTRL   vlan_tag_ctrl;
    VLAN_CFG    vlan_cfg;
}PPA_CMD_BR_IF_VLAN_CFG;

//ioctl parameters
typedef struct {
    union {
        //PPA_IFNAME    *ifname;
        uint32_t        ip_src;
        uint32_t        eth_protocol;
        uint32_t        ingress_vlan_tag;
    }           match_field;
    uint32_t    is_ifname;
    char        ifname[16];
    uint32_t    match_flags;
} PPA_IOCTL_VLAN_MATCH_FIELD;

typedef struct {
    char        ifname[16];
    uint32_t    if_flags;
} PPA_IOCTL_IFINFO;


typedef struct {
    uint16_t            vlan_vci;
    uint32_t            num_ifs;
    PPA_IOCTL_IFINFO    vlan_if_membership;
} PPA_IOCTL_VLAN_INFO;


typedef struct {
    PPA_IOCTL_VLAN_MATCH_FIELD  match_field;
    PPA_IOCTL_VLAN_INFO         vlan_info;
} PPA_IOCTL_VLAN_FILTER_CONFIG;


typedef struct{
    uint32_t    port;
    uint8_t     mac[PPA_ETH_ALEN];
    uint32_t    vci;
    uint32_t    f_src_mac_drop;
    uint32_t    f_vlan_ins_enable;
    uint32_t    new_vci;
    uint32_t    f_vlan_rm_enable;
    uint32_t    dest_list;
    uint32_t    *p_entry;
} PPA_IOCTL_BR_ENTRY;


typedef struct {
    uint32_t    ig_criteria_type;
    uint32_t    ig_criteria;
    uint32_t    new_vci;
    uint32_t    vlan_port_map;
} PPA_IOCTL_VLAN_MAP_ENTRY;


typedef struct
{
    uint32_t    vlan_p;
    uint32_t    queue;
} PPA_IOCTL_BR_VLAN_PRIO;



/*
 * ####################################
 *           Inline Functions
 * ####################################
 */



/*
 * ####################################
 *             Declaration
 * ####################################
 */

#ifdef __KERNEL__
  int32_t ifx_ppa_init(PPA_INIT_INFO *, uint32_t);
  void ifx_ppa_exit(void);

  int32_t ifx_ppa_enable(uint32_t, uint32_t, uint32_t);
  int32_t ifx_ppa_get_status(uint32_t *, uint32_t *);

  //int32_t ifx_ppe_directpath_register_dev(uint32_t, PPA_NETIF *, uint32_t);
  //int32_t ifx_ppe_directpath_register_rx_callback(uint32_t, PPE_FP_RX_FN, uint32_t);
  //int32_t ifx_ppe_directpath_send(PPA_NETIF *, PPA_BUF *, uint32_t);

  int32_t ifx_ppa_session_add(PPA_BUF *, PPA_SESSION *, uint32_t);
  int32_t ifx_ppa_session_delete(PPA_SESSION *, uint32_t flags);
  int32_t ifx_ppa_session_modify(PPA_SESSION *, PPA_SESSION_EXTRA *, uint32_t);
  int32_t ifx_ppa_session_get(PPA_SESSION ***, PPA_SESSION_EXTRA **, int32_t *, uint32_t);

 #if !defined(USE_NEW_MC_API) || !USE_NEW_MC_API
  int32_t ifx_ppa_mc_session_add(PPA_BUF *, PPA_MC_SESSION *, uint32_t);
  int32_t ifx_ppa_mc_session_delete(uint32_t, uint32_t);
  int32_t ifx_ppa_mc_session_modify(uint32_t, PPA_MC_SESSION *, PPA_SESSION_EXTRA *, uint32_t);
  int32_t ifx_ppa_all_mc_session_get(PPA_MC_SESSION ***, PPA_SESSION_EXTRA **, int32_t *, uint32_t);
  int32_t ifx_ppa_mc_session_get(uint32_t, PPA_MC_SESSION **, PPA_SESSION_EXTRA *, uint32_t);
 #else
  int32_t ifx_ppa_mc_group_update(PPA_MC_GROUP *, uint32_t);
  int32_t ifx_ppa_mc_group_get(IPADDR, PPA_MC_GROUP *, uint32_t);
 #endif

  int32_t ifx_ppa_set_session_inactivity(PPA_U_SESSION *, int32_t);
  int32_t ifx_ppa_inactivity_status(PPA_U_SESSION *);

  int32_t ifx_ppa_get_if_stats(PPA_IFNAME *, PPA_IF_STATS *, uint32_t);
  int32_t ifx_ppa_get_accel_stats(PPA_ACCEL_STATS *, PPA_ACCEL_STATS *, uint32_t);

 #if 0
  int32_t ifx_ppa_filter_add(PPA_FILTER *, PPA_ACTION *, uint32_t);
 #endif

  int32_t ifx_ppa_set_if_mac_address(PPA_IFNAME *, uint8_t *);
  int32_t ifx_ppa_get_if_mac_address(PPA_IFNAME *, uint8_t *);

  int32_t ifx_ppa_bridge_entry_add(uint8_t *, PPA_NETIF *, uint32_t);
  int32_t ifx_ppa_bridge_entry_delete(uint8_t *, uint32_t);
  int32_t ifx_ppa_set_bridge_entry_timeout(uint8_t *, uint32_t);
  int32_t ifx_ppa_bridge_entry_inactivity_status(uint8_t *);
  int32_t ifx_ppa_bridge_entry_hit_time(uint8_t *, uint32_t *);

  int32_t ifx_ppa_set_bridge_if_vlan_config(PPA_NETIF *netif, VLAN_TAG_CTRL *, VLAN_CFG *, uint32_t);
  int32_t ifx_ppa_get_bridge_if_vlan_config(PPA_NETIF *netif, VLAN_TAG_CTRL *, VLAN_CFG *, uint32_t);
  int32_t ifx_ppa_vlan_filter_add(PPA_VLAN_MATCH_FIELD *, PPA_VLAN_INFO *, uint32_t);
  int32_t ifx_ppa_vlan_filter_del(PPA_VLAN_MATCH_FIELD *, uint32_t);

  int32_t ifx_ppa_add_itf(PPA_IFINFO *p_ifs, uint32_t is_lanif);

  int32_t ppa_bridging_lookup_session(uint8_t *, PPA_NETIF *, struct bridging_session_list_item **);
  int32_t ppa_lookup_mc_session(uint32_t, struct session_list_item **);
  int32_t ppa_lookup_session(PPA_SESSION *p_session, uint32_t is_reply, struct session_list_item **pp_item);

  int32_t ppa_is_pkt_type_supported_for_accel (PPA_BUF *ppa_buf);
  int32_t ppa_send_pkt_to_accel_hw (PPA_BUF *ppa_buf, uint32_t rx_if_id, uint32_t tx_dma_qid, int32_t len);
  
#endif

//#include <asm/amazon_se/ifx_ppa_hook.h>



#endif  //  __DANUBE_PPA_API_H__20060913_1556__
